Imports Palisade.RdkPia

Public Class Form1

    Dim WithEvents myRDKApp As RDKApplication
    ' store inputs and outputs in arrays for ease of reference
    Dim priceInputs() As RDKInput
    Dim costInputs() As RDKInput
    Dim amtSoldInputs() As RDKInput
    Dim regionProfits() As RDKOutput

    Private Const TWIPS_PER_PIXEL As Int32 = 15

    Private Sub ButtonSimulate_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles ButtonSimulate.Click

        myRDKApp = New RDKApplication
        myRDKApp.Initialize(False)

        AddInputs()

        AddOutputs()

        ' define simulation settings
        With myRDKApp.SimSettings
            .NumIterations = 2000
        End With

        myRDKApp.Simulate()

        DisplayOutputStatistics()

        GraphOutputs()

        myRDKApp.Free()
    End Sub

    Private Sub AddInputs()

        Dim i As Long
        Dim distr As RDKDistribution

        ReDim priceInputs(2)
        ReDim costInputs(2)
        ReDim amtSoldInputs(2)

        For i = 1 To 3
            distr = GetLognormalTruncatedDistribution(Me.Controls("Price" & i & 1).Text, Me.Controls("Price" & i & 2).Text, Me.Controls("Price" & i & 3).Text, Me.Controls("Price" & i & 4).Text)
            priceInputs(i - 1) = myRDKApp.Inputs.Add("Price" & i, distr)
        Next

        For i = 1 To 3
            distr = GetNormalTruncatedDistribution(Me.Controls("Cost" & i & 1).Text, Me.Controls("Cost" & i & 2).Text, Me.Controls("Cost" & i & 3).Text, Me.Controls("Cost" & i & 4).Text)
            costInputs(i - 1) = myRDKApp.Inputs.Add("Cost" & i, distr)
        Next

        For i = 1 To 2
            distr = GetTriangularDistribution(Me.Controls("amt" & i & 1).Text, Me.Controls("amt" & i & 2).Text, Me.Controls("amt" & i & 3).Text)
            amtSoldInputs(i - 1) = myRDKApp.Inputs.Add("AmtSold" & i, distr)
        Next

        distr = GetTrigenDistribution(Me.Controls("amt31").Text, Me.Controls("amt32").Text, Me.Controls("amt33").Text, Me.LowerPercentile.Text, Me.UpperPercentile.Text)
        amtSoldInputs(2) = myRDKApp.Inputs.Add("AmtSold3", distr)

    End Sub

    Private Sub AddOutputs()
        Dim i As Long
        ReDim regionProfits(2)
        For i = 1 To 3
            regionProfits(i - 1) = myRDKApp.Outputs.Add("Region" & i)
        Next
        myRDKApp.Outputs.Add("Total Profit")
    End Sub

    Private Function GetLognormalTruncatedDistribution(ByVal mean As Double, ByVal stdDev As Double, ByVal min As Double, ByVal max As Double) As RDKDistribution
        Dim distr As RDKDistribution
        distr = New RDKDistribution
        With distr
            .DistType = RDKDistributionType.RDKDistTypeLOGNORM
            .Arg(1) = mean
            .Arg(2) = stdDev
            .TruncMin = min
            .TruncMax = max
        End With
        GetLognormalTruncatedDistribution = distr
    End Function

    Private Function GetNormalTruncatedDistribution(ByVal mean As Double, ByVal stdDev As Double, ByVal min As Double, ByVal max As Double) As RDKDistribution
        Dim distr As RDKDistribution
        distr = New RDKDistribution
        With distr
            .DistType = RDKDistributionType.RDKDistTypeNORMAL
            .Arg(1) = mean
            .Arg(2) = stdDev
            .TruncMin = min
            .TruncMax = max
        End With
        GetNormalTruncatedDistribution = distr
    End Function

    Private Function GetTriangularDistribution(ByVal min As Double, ByVal ml As Double, ByVal max As Double) As RDKDistribution
        Dim distr As RDKDistribution
        distr = New RDKDistribution
        With distr
            .DistType = RDKDistributionType.RDKDistTypeTRIANG
            .Arg(1) = min
            .Arg(2) = ml
            .Arg(3) = max
        End With
        GetTriangularDistribution = distr
    End Function


    Private Function GetTrigenDistribution(ByVal min As Double, ByVal ml As Double, ByVal max As Double, ByVal lowerPerc As Double, ByVal upperPerc As Double) As RDKDistribution
        Dim distr As RDKDistribution
        distr = New RDKDistribution
        With distr
            .DistType = RDKDistributionType.RDKDistTypeTRIGEN
            .Arg(1) = min
            .Arg(2) = ml
            .Arg(3) = max
            .Arg(4) = lowerPerc
            .Arg(5) = upperPerc
        End With
        GetTrigenDistribution = distr
    End Function

    Private Sub myRDKApp_Iteration(ByRef whichSim As Integer, ByRef whichIter As Integer, ByRef cancel As Boolean) Handles myRDKApp.Iteration
        Dim i As Long
        Dim revenue As Double
        Dim cost As Double
        Dim totalProfit As Double
        For i = 1 To 3
            revenue = amtSoldInputs(i - 1).Value * priceInputs(i - 1).Value
            cost = amtSoldInputs(i - 1).Value * costInputs(i - 1).Value
            regionProfits(i - 1).Value = revenue - cost
            totalProfit = totalProfit + regionProfits(i - 1).Value
        Next
        myRDKApp.Outputs("Total Profit").Value = totalProfit
    End Sub

    Private Sub DisplayOutputStatistics()
        With Me.myRDKApp.Outputs("Total Profit").Results
            Me.TextBoxTotalProfitMean.Text = .Statistics.Mean.ToString("0")
            Me.TextBoxTotalProfitStdDev.Text = .Statistics.StdDeviation.ToString("0")
        End With
    End Sub

    Private Sub GraphOutputs()

        ' graph output
        ' first create file with graph
        Dim tempFilePath As String
        With myRDKApp.GraphDefaults
            .Destination = RDKGraphDestination.RDKJPGFile
            ' this line makes a temp file
            tempFilePath = IO.Path.GetTempFileName
            ' I don't want a file, just a path ending in .jpg
            Kill(tempFilePath)
            tempFilePath = tempFilePath & ".jpg"
            .DestinationFile = tempFilePath
            .PictureHeight = Me.PictureBoxGraph.Height * TWIPS_PER_PIXEL
            .PictureWidth = Me.PictureBoxGraph.Width * TWIPS_PER_PIXEL
            .DisplayMean = True
        End With
        With myRDKApp
            .Outputs("Total Profit").Results.Graph(RDKResultCurveType.RDKResultCurveTypeCumulativeAscending, .Outputs(1).Results, .Outputs(2).Results, .Outputs(3).Results)
        End With

        Me.PictureBoxGraph.Image = New Bitmap(tempFilePath)

    End Sub
End Class
